/*
 * Reads a polyhedron file on stdin with rationals and outputs an equivalent 
 * system with integer  coefficents.
 * 
 * David Bremner. bremner@unb.ca
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "process_args.h"
#include "ine_io.h"

#ifndef LRSMP
#error This file only compiles with LRSMP
#endif

#include "lrsmp.h"
#include "rat2int.h"

static   int  m,n,type;
#define MP_DIGITS 1000L

static void lcm3(lrs_mp a,lrs_mp b,lrs_mp c);
static void mul_row(lrs_mp *N, lrs_mp *D, lrs_mp l, int offset);
static void div_row(lrs_mp *N, lrs_mp *D, lrs_mp g, int offset);

int
main(int argc, char **argv)
{


  int i,j;
  lrs_mp l,g,scratch;
  char *type;
  lrs_mp *D, *N;
  char *ok[]={ "integer", "rational"};
  process_args(argc,argv,DOCSTRING);
  lrs_mp_init (MP_DIGITS,stdin,stdout);   
  scan_for_begin(&m,&n,&type);

  check_type(type, ok , 2);

  printf("%d %d integer\n",m,n);
  D=calloc(n,sizeof(lrs_mp));
  N=calloc(n,sizeof(lrs_mp));
  
  for (i=0;i<m;i++)    {
  
      for (j=0; j<n; j++){
	readrat(N[j],D[j]);
      }

      lcm3(D[0],D[1],l);
      
      for(j=2;j<n;j++){
	lcm3(l,D[j],l);
      }
      
      mul_row(N,D,l,0);

      /* reinitialize gcd */
      copy(g,N[0]);
      copy(scratch,N[1]);
      gcd(g,scratch);

      for (j=2; j<n ; j++){
      	copy(scratch,N[j]);
	gcd(g,scratch);
      }

      if (negative(g)) 
	storesign(g,POS);
      
      div_row(N,D,g,0);

      for (j=0; j< n; j++){
	pmp("",N[j]);
      }
      fputs("\n",stdout);
  }
  
  printf("end\n");

}

/* c<- lcm(a,b) */

static void lcm3(lrs_mp a,lrs_mp b,lrs_mp c)
{
  lrs_mp gc,ab,scratch;
  
  copy(gc,a);
  copy(scratch,b);
  gcd(gc,scratch);
  

  mulint(a,b,ab);
    
  divint(ab,gc,c);
    
}


static void
mul_row(lrs_mp *N, lrs_mp *D, lrs_mp l, int offset)
{
    lrs_mp  factor,scratch;
  int j;

  for  (j=offset;j<n;  j++) {

    /* 
     * Now multiply each numerator n_i by lcm/d_i.
     * Implicitly, each denominator is now lcm
     */

    copy(scratch,l);
    divint(scratch,D[j],factor);
    mulint(N[j],factor,N[j]);
  }

}      

static void
div_row(lrs_mp *N, lrs_mp *D, lrs_mp g, int offset)
{
  lrs_mp  scratch;
  int j;

  for  (j=0;j<n;  j++)
    {
      copy(scratch,N[j]);
	
      divint(scratch,g,N[j]);
    }

}

